# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2019 Nexedi SA and Contributors. All Rights Reserved.
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
#
##############################################################################

from erp5.component.test.testSlapOSPDMSkins import TestSlapOSPDMMixinSkins

class TestSlapOSPDMCreateUpgradeDecisionSkins(TestSlapOSPDMMixinSkins):

  def test_createUpgradeDecision_destroyed_instance(self):
    instance_tree = self.portal.instance_tree_module.newContent(
      portal_type="Instance Tree"
    )
    self.portal.portal_workflow._jumpToStateFor(instance_tree,
                                                'destroy_requested')
    self.assertEqual(None, instance_tree.InstanceTree_createUpgradeDecision())

  def test_createUpgradeDecision_without_softwareProduct(self):
    instance_tree = self.portal.instance_tree_module.newContent(
      portal_type="Instance Tree"
    )
    self.assertEqual(None, instance_tree.InstanceTree_createUpgradeDecision())

  def bootstrapAllocableInstanceTree(self, is_allocated=False):
    project = self.addProject()
    person = self._makePerson(project)
    software_product = self._makeSoftwareProduct(project)
    release_variation = software_product.contentValues(portal_type='Software Product Release Variation')[0]
    type_variation = software_product.contentValues(portal_type='Software Product Type Variation')[0]

    self.tic()
    person.requestComputeNode(compute_node_title='test compute node',
                              project_reference=project.getReference())
    self.tic()
    compute_node = self.portal.portal_catalog.getResultValue(
      portal_type='Compute Node',
      reference=self.portal.REQUEST.get('compute_node_reference')
    )
    assert compute_node is not None

    request_kw = dict(
      software_release=release_variation.getUrlString(),
      software_type=type_variation.getTitle(),
      instance_xml=self.generateSafeXml(),
      sla_xml=self.generateSafeXml(),
      shared=False,
      software_title='test tree',
      state='started',
      project_reference=project.getReference()
    )
    person.requestSoftwareInstance(**request_kw)
    instance_tree = self.portal.REQUEST.get('request_instance_tree')

    if is_allocated:
      partition = compute_node.newContent(portal_type='Compute Partition')
      instance = instance_tree.getSuccessorValue()
      instance.edit(aggregate_value=partition)

    self.tic()
    return software_product, release_variation, type_variation, compute_node, instance_tree

  ##########################################################################
  # Not allocated
  ##########################################################################
  def test_createUpgradeDecision_notAllocated_newRelease(self):
    software_product, _, type_variation, compute_node, instance_tree = self.bootstrapAllocableInstanceTree()

    new_release_variation = self._makeSoftwareRelease(software_product)
    self.addAllocationSupply("for compute node 2", compute_node, software_product,
                             new_release_variation, type_variation)
    self.tic()

    upgrade_decision = instance_tree.InstanceTree_createUpgradeDecision()
    self.assertEqual('started', upgrade_decision.getSimulationState())

  def test_createUpgradeDecision_notAllocated_sameRelease(self):
    software_product, release_variation, type_variation, compute_node, instance_tree = self.bootstrapAllocableInstanceTree()

    self.addAllocationSupply(
      "for compute node", compute_node, software_product,
      release_variation, type_variation)

    self.assertEqual(None, instance_tree.InstanceTree_createUpgradeDecision())

  def test_createUpgradeDecision_notAllocated_twoRelease(self):
    software_product, release_variation, type_variation, compute_node, instance_tree = self.bootstrapAllocableInstanceTree()

    self.addAllocationSupply(
      "for compute node", compute_node, software_product,
      release_variation, type_variation)
    new_release_variation = self._makeSoftwareRelease(software_product)
    self.addAllocationSupply("for compute node 2", compute_node, software_product,
                             new_release_variation, type_variation)
    self.tic()

    self.assertEqual(None, instance_tree.InstanceTree_createUpgradeDecision())

  def test_createUpgradeDecision_notAllocated_newRelease_ongoingDecision(self):
    software_product, _, type_variation, compute_node, instance_tree = self.bootstrapAllocableInstanceTree()

    new_release_variation = self._makeSoftwareRelease(software_product)
    self.addAllocationSupply("for compute node 2", compute_node, software_product,
                             new_release_variation, type_variation)
    self.tic()

    upgrade_decision = instance_tree.InstanceTree_createUpgradeDecision()
    self.assertEqual('started', upgrade_decision.getSimulationState())
    self.tic()

    self.assertEqual(None, instance_tree.InstanceTree_createUpgradeDecision())

  def test_createUpgradeDecision_notAllocated_newRelease_ongoingDecisionActivity(self):
    software_product, _, type_variation, compute_node, instance_tree = self.bootstrapAllocableInstanceTree()

    new_release_variation = self._makeSoftwareRelease(software_product)
    self.addAllocationSupply("for compute node 2", compute_node, software_product,
                             new_release_variation, type_variation)
    self.tic()

    upgrade_decision = instance_tree.InstanceTree_createUpgradeDecision()
    self.assertEqual('started', upgrade_decision.getSimulationState())
    self.commit()

    self.assertEqual(None, instance_tree.InstanceTree_createUpgradeDecision())

  def test_createUpgradeDecision_notAllocated_newRelease_cancelledDecision(self):
    software_product, _, type_variation, compute_node, instance_tree = self.bootstrapAllocableInstanceTree()

    new_release_variation = self._makeSoftwareRelease(software_product)
    self.addAllocationSupply("for compute node 2", compute_node, software_product,
                             new_release_variation, type_variation)
    self.tic()

    upgrade_decision = instance_tree.InstanceTree_createUpgradeDecision()
    self.assertEqual('started', upgrade_decision.getSimulationState())
    upgrade_decision.reject()
    self.tic()

    upgrade_decision2 = instance_tree.InstanceTree_createUpgradeDecision()
    self.assertEqual('started', upgrade_decision2.getSimulationState())

  ##########################################################################
  # Allocated on Compute Node
  ##########################################################################
  def test_createUpgradeDecision_allocatedOnComputeNode_newReleaseOnAnotherComputeNode(self):
    software_product, _, type_variation, _, instance_tree = self.bootstrapAllocableInstanceTree(is_allocated=True)

    project = instance_tree.getFollowUpValue()
    person = instance_tree.getDestinationSectionValue()

    person.requestComputeNode(compute_node_title='another test compute node',
                              project_reference=project.getReference())
    self.tic()
    compute_node2 = self.portal.portal_catalog.getResultValue(
      portal_type='Compute Node',
      reference=self.portal.REQUEST.get('compute_node_reference')
    )
    assert compute_node2 is not None

    new_release_variation = self._makeSoftwareRelease(software_product)
    self.addAllocationSupply("for compute node 2", compute_node2, software_product,
                             new_release_variation, type_variation)
    self.tic()

    self.assertEqual(None, instance_tree.InstanceTree_createUpgradeDecision())

  def test_createUpgradeDecision_allocatedOnComputeNode_newReleaseOnComputeNode(self):
    software_product, _, type_variation, compute_node, instance_tree = self.bootstrapAllocableInstanceTree(is_allocated=True)

    new_release_variation = self._makeSoftwareRelease(software_product)
    self.addAllocationSupply("for compute node", compute_node, software_product,
                             new_release_variation, type_variation)
    self.tic()

    upgrade_decision = instance_tree.InstanceTree_createUpgradeDecision()
    self.assertEqual('started', upgrade_decision.getSimulationState())

  def test_createUpgradeDecision_allocatedOnComputeNode_twoRelease(self):
    software_product, release_variation, type_variation, compute_node, instance_tree = self.bootstrapAllocableInstanceTree(is_allocated=True)

    self.addAllocationSupply(
      "for compute node", compute_node, software_product,
      release_variation, type_variation)
    new_release_variation = self._makeSoftwareRelease(software_product)
    self.addAllocationSupply("for compute node 2", compute_node, software_product,
                             new_release_variation, type_variation)
    self.tic()

    self.assertEqual(None, instance_tree.InstanceTree_createUpgradeDecision())
